%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This work is licensed under a Creative Commons Attribution 4.0
% International License. http://creativecommons.org/licenses/by/4.0/
% The authors do not assume liability for errors contained in or for
% damages arising from the use of the software.
%
% CC-BY (4.0) Oliver Taubmann (oliver.taubmann@fau.de), 2015
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This script showcases interactive comparison of two images.

%% Settings

segmentation_file = 'segmentation.mat'; % Loaded if it exists, saved to otherwise

prof_filter_size = 5;            % Size of filter kernel for smoothing profiles
es_filter_size   = 20;           % Size of filter kernel for smoothing ES values
prof_spacing     = 0.5;          % Spacing of profiles in voxels
prof_len         = 5;            % Length of profiles in voxels
num_samples      = prof_len * 4; % Number of samples per profile

%% Load Images

titles = {'Reference', 'Compared'};

% (May be replaced by other file format loaders as needed)
volume1 = read_mhd('Path/To/ReferenceImage');
volume2 = read_mhd('Path/To/ComparedImage');

% Create segmentation if it doesn't exist
if ~exist(segmentation_file, 'file')
    
    disp('Segmentation file not found, starting manual segmentation');
    
    z_index = [];
    
    while isempty(z_index)
        answer = inputdlg('Which z-slice do you want to segment? (1-based index)', 'Segmentation');
        z_index = round(str2num(answer{1}));
        
        if isempty(z_index)
            fprintf(2, 'Index is not numeric, try again!\n');
            continue;
        end
        if ~(z_index >= 1 && z_index <= volume1.size(3))
            fprintf(2, 'Index is out of valid range [%d..%d], try again!\n', 1, volume1.size(3));
            z_index = [];
        end
    end
    
    slice1 = double(volume1.data(:, :, z_index)');
    slice2 = double(volume2.data(:, :, z_index)');
    slices = {slice1, slice2};
    
    figure(1);
    hold off;
    
    for i = 1:2
        subplot(1, 2, i);
        imagesc(slices{i});
        colormap(gray);
        axis('image');
        hold on;
        title(titles{i});
        axis off;
    end
    
    [xs, ys] = segment_edge();
    
    close;
    
    save(segmentation_file, 'xs', 'ys', 'z_index');
end

load(segmentation_file, 'xs', 'ys', 'z_index');

slice1 = double(volume1.data(:, :, z_index)');
slice2 = double(volume2.data(:, :, z_index)');

slices = {slice1, slice2};

%% Plot both slices

figure(1);
hold off;

for i = 1:2
    subplot(1, 2, i);
    imagesc(slices{i});
    colormap(gray);
    axis('image');
    hold on;
    title(titles{i});
    axis off;
end

%% Plot segmentation in both slices

for i = 1:2
    subplot(1, 2, i);
    plot(xs, ys, 'r', 'LineWidth', 0.75);
end

%% Create profiles for segmentation

[prof_ps, ps] = create_profiles(xs, ys, prof_len, prof_spacing);

%% Plot profiles in both slices

for i = 1:2
    subplot(1, 2, i);
    for j = 1:size(prof_ps)/2
        pp1 = 2*j-1;
        pp2 = 2*j;
        plot(prof_ps(pp1:pp2, 1), prof_ps(pp1:pp2, 2), 'y', 'LineWidth', 0.25);
    end
end

%% Sample profiles

profs = cell(1, 2);

for i = 1:2
    profs{i} = sample_profiles(prof_ps, slices{i}, num_samples);
    
    % Remove outliers
    profs{i} = medfilt2(profs{i}, [1 prof_filter_size], 'symmetric'); 
end

%% Compute edge sharpness on profiles

ess       = cell(1, 2);
ess_filt  = cell(1, 2);
median_es = cell(1, 2);

for i = 1:2
    ess{i} = compute_es(profs{i}, prof_len);
    
    % medfilt1 is in signal processing toolbox...
	% so we 'abuse' medfilt2 from image proc tb :)
    ess_filt{i} = medfilt2(ess{i}, [es_filter_size, 1], 'symmetric'); 
    
    median_es{i} = median(ess{i});
end

%% Plot edge sharpness

figure(2);
h = plot(ess_filt{1}, 'r');
hold on;
plot(ess_filt{2}, 'b');
legend('Reference Image', 'Compared Image', 'Location', 'SouthEast');
title(sprintf('Edge Sharpness (Ref. = %.7f, Cmp. = %.7f)', median_es{1}, median_es{2}));
xlabel('Profiles');
ylabel('ES');

hold off;
saveas(h, 'es_plot.emf');

esi = ess_filt{2} ./ ess_filt{1};

% Plot both slices again without any overlays for comparison

figure(3);

for i = 1:2
    subplot(1, 2, i);
    imagesc(slices{i});
    colormap(gray);
    axis('image');
    hold on;
    %title(titles{i});
    axis off;
end

hold off;

% Compute colors from esi values
cm_len = 100;
cm = redblue(cm_len);
cm_indices = 1 + ((esi - 1) ./ max(abs(esi - 1))); % scale to [0..2]
cm_indices = cm_indices .* cm_len/2; % scale such that esi=1 -> middle of cm
cm_indices = max(min(1 + floor(cm_indices), cm_len), 1); % discretize and clamp to [1..cm_len]
cols = cm(cm_indices, :);

% Plot both slices again for ESI visualization

figure(4);

for i = 1:2
    subplot(1, 2, i);
    imagesc(slices{i});
    colormap(gray);
    axis('image');
    hold on;
    title(titles{i});
    axis off;
end

% In the first image, plot segmentation

subplot(1, 2, 1);
plot(xs, ys, 'r', 'LineWidth', 0.75);
hold off;

% In the second image, plot ESI

subplot(1, 2, 2);
hold on;

for i = 1:size(ps, 1)
	plot(ps(i, 1), ps(i, 2), 'x', 'Color', cols(i, :));
end

hold off;